import produce from 'immer';
import { applyMiddleware, combineReducers, createStore } from 'redux';
import { composeWithDevTools } from 'redux-devtools-extension';
import { NAME_SPACE_FLAG, REDUCER_KEY } from './constant';
import { getKey } from './utils';
export function createModule(options) {
    return options;
}
let _store;
export const _actionMap = {};
export const selectors = {};
export function generateActionMap(moduleName, actionName, actionNameWithNamespace) {
    _actionMap[moduleName] = {
        ..._actionMap[moduleName],
        [actionName]: actionNameWithNamespace,
    };
}
function enhanceReducerModule(params) {
    const { state, action, reducer, namespace = '' } = params;
    return Object.keys(reducer)
        .map((key) => namespace + NAME_SPACE_FLAG + key)
        .includes(action.type)
        ? produce(state, (draft) => {
            return reducer[getKey(action.type)](action.payload, draft);
        })
        : state;
}
function createReducerModule(reducerModuleConfig) {
    const { reducer, namespace } = reducerModuleConfig;
    const reducerModule = (state = reducerModuleConfig.state, action) => enhanceReducerModule({
        state,
        action,
        reducer,
        namespace,
    });
    reducerModule[REDUCER_KEY] = getActionMap(reducer, namespace);
    return reducerModule;
}
function getActionMap(reducerModule, namespace) {
    return Object.keys(reducerModule).reduce((actionMap, actionName) => {
        const actionNameWithNamespace = namespace + NAME_SPACE_FLAG + actionName;
        generateActionMap(namespace, actionName, actionNameWithNamespace);
        return {
            ...actionMap,
            [actionName]: (payload) => {
                _store.dispatch({
                    type: actionNameWithNamespace,
                    payload,
                });
            },
        };
    }, {});
}
export function mountReducerModules(store, reducerModules) {
    _store = store;
    Object.keys(reducerModules).forEach((moduleName) => {
        _store[moduleName] = reducerModules[moduleName][REDUCER_KEY];
    });
}
function generateReducerMap(reducersToCombine) {
    const reducerMap = {};
    Object.keys(reducersToCombine).forEach((moduleName) => {
        reducerMap[moduleName] = reducersToCombine[moduleName][REDUCER_KEY];
    });
    return reducerMap;
}
function processReducerModules(reducerModules) {
    const reducersToCombine = {};
    Object.keys(reducerModules).forEach((reducerName) => {
        const moduleSelectors = reducerModules[reducerName].selector;
        if (moduleSelectors) {
            Object.keys(moduleSelectors).forEach((key) => (selectors[key] = () => moduleSelectors[key](_store.getState())));
        }
        reducersToCombine[reducerName] = createReducerModule(reducerModules[reducerName]);
    });
    const reducerMap = generateReducerMap(reducersToCombine);
    return {
        reducersToCombine,
        reducerMap,
    };
}
export function run(options) {
    const { modules, middlewares = [] } = options;
    const { reducersToCombine, reducerMap } = processReducerModules(modules);
    const rootReducer = combineReducers(reducersToCombine);
    const store = createStore(rootReducer, composeWithDevTools(applyMiddleware(...middlewares)));
    mountReducerModules(store, reducersToCombine);
    return {
        store,
        selectors,
        reducers: reducerMap,
        effects: {},
        actions: _actionMap,
    };
}
